/*-------------------------------------------------------------
 *  Copyright (c) 2010-2015, MSS ComBlock. All rights reserved.
 *
 * 	Filename: initConfig.c
 *  Last modified: 12/12/15
 *  Authors:
 *  	Alain Zarembowitch / MSS ComBlock
 *  Version: 1.3
 *  Inheritance: CB1600_2_00 1/10/11
 *  Description: initializes the LPC1759 internal configuration (or re-configure it)
 *  within the COM-4103 PA+LNA
 *  No configuration of external peripherals here.
 *
 *  Dependencies/Pre-requisite: N/A
 -------------------------------------------------------------*/
#include "main.h"
#include "initConfig.h"
#include "utils.h"

const unsigned crp __attribute__((section(".ARM.__at_0x1FC"))) = NOCRP;
  // no code read protection (change as needed. Beware of unrecoverable CRP3)
  // CRP is located at address 0x1FC with the assistance of the .ld linker

void initConfig (void)
{

	//--------------------------------------------------------------------------------
	// Power Control for Peripherals register
	PCONP |= 0x82000000;
		// enable the following subsystems (disabled after reset)
		// bit 25 PCUART3 UART 3 power/clock control bit.
		// bit 31 PCUSB USB interface power/clock control bit.
	PCONP &= ~0x04080680;
		// disable the following subsystems (enabled by default after reset)
		// bit 7 PCI2C0 The I2C0 interface power/clock control bit.
		// bit 9 RTC clock
		// bit 10 PCSSP1 The SSP 1 interface power/clock control bit.	// overruled in main.c
		// bit 19 PCI2C1 The I2C1 interface power/clock control bit.
		// bit 26 PCI2C2 I2C interface 2 power/clock control bit.



	//--------------------------------------------------------------------------------
	// UM10360 Chapter 8: LPC17xx Pin connect block
	// PINSEL  00 = primary function, 01 = first alternate function, 10 = 2nd alternate, 11 = third alternate.
	PINSEL0 = 0x00000050;
		// all primary functions except the following:
		// p0[2] = uart tx
		// p0[3] = uart rx
		// SSP defined in main.c
	PINSEL1 = 0x14240000;
		// all primary functions except the following:
		// p0[25] = AD0[2]
		// p0[26] = AOUT
		// p0[29] = USB_D+
		// p0[30] = USB_D-
	PINSEL2 = 0x00000000;
		// all primary functions except the following:
	PINSEL3 = 0xE0000020;
		// all primary functions except the following:
		// p1[18] = PWM1.1
		// p1[30] = USB VBUS
		// p1[31] = AD0[5]
	PINSEL4 = 0x00000000;
		// all primary functions except the following:
	PINSEL7 = 0x00000000;
		// all primary functions except the following:
	PINSEL9 = 0x0F000000;
		// all primary functions except the following:
		// p4[25] = TXD3
		// p4[26] = RXD3
	PINSEL10 = 0x00000000;
		// trace function disabled

	// PINMODE 00 = pull-up, 01 = repeater mode, 10 = no pullup/down, 11 = pull-down
	PINMODE0 = 0x00000000;	// all pull-ups (default) except
	PINMODE1 = 0x00080000;	// all pull-ups (default) except
		// P0[25] ADC has no PU/PD
	PINMODE2 = 0x00000000;	// all pull-ups (default) except
	PINMODE3 = 0xB0000220; // all pull-ups except the following:
		// P1[18] PWM has no PU/PD
		// P1[20] PWM has no PU/PD
		// pull-down on the VBUS pin otherwise VBUS pin may be staying high for a long time after a cable is removed. See IAI V5 board
		// P1[31] ADC has no PU/PD
	//PINMODE4 = 0x00000000;	// all pull-ups (default)
	//PINMODE7 = 0x00000000;	// all pull-ups (default)
	//PINMODE9 = 0x00000000;	// all pull-ups (default)

	// PINMODE_OD  0 = normal, 1 = open drain
	//PINMODE_OD0 = 0x00000000;	// all normal, no open drain
	//PINMODE_OD1 = 0x00000000;	// all normal, no open drain
	//PINMODE_OD2 = 0x00000000;	// all normal, no open drain
	//PINMODE_OD3 = 0x00000000;	// all normal, no open drain
	//PINMODE_OD4 = 0x00000000;	// all normal, no open drain

	// I2C Pin Configuration register (I2CPADCFG)
	//I2CPADCFG = 0x00000000;  // no I2C used here -> default

	//--------------------------------------------------------------------------------
	// UM10360	Chapter 9: LPC17xx General Purpose Input/Output (GPIO)

	// FIOxDIR  port direction. 0 = input, 1 = output
	FIO0DIR = 0x04000004;
		// outputs = TXD0, AOUT
	FIO1DIR = 0x00C40313;
		// outputs = TX_EN2, RFOUT_ENB, RX_TXN2, TX_RXN2, LNA_ENB,
		// PWM1_1,USB_CONNECTB, RED LED
	FIO2DIR = 0x00000000;
		// outputs = N/A
	FIO4DIR = 0x10000000;
		// outputs = right TX

	// FIOxPIN Set output bits
	FIO0PIN = 0x00000004;
		// serial tx UART inactive (high)
	FIO1PIN = 0x00C00010;
		// RX_TXN2, USB disconnect, Red led (temporary)
	FIO2PIN = 0x00000000;
	FIO4PIN = 0x10000000;
		//  serial right tx UART inactive (high)

	// mask pins not to be used
	//FIO2MASK = 0xFFFFF800;

	//--------------------------------------------------------------------------------
	// UM10360 Chapter 4: LPC17xx Clocking and power control
	//clock_power_profile(1);  // RTC clock, 120 MHz CPU, USB
	//clock_power_profile(2);  // 20 MHz clock, 120 MHz CPU, USB
	//clock_power_profile(3);  // 25 MHz clock, 120 MHz CPU, USB
	//clock_power_profile(4);  // 13 MHz clock, 120 MHz CPU, USB
	//clock_power_profile(5);  // 19.2 MHz clock, 120 MHz CPU, USB
	clock_power_profile(6);  // 24 MHz clock, 120 MHz CPU, USB

	//--------------------------------------------------------------------------------
	// UM10360 Chapter 11: LPC17xx USB Device Controller
	usb_softconnect = 0;
	USB_connect_ctrl();

	return;
}

void clock_power_profile(char profile_no){
	int i;

	interruptsEnable(0);	// disable interrupts

	// select one among several clock/power profiles
	if(profile_no == 1){
		// profile 1:  PLL0 clock source = 32.768 KHz RTC clock
		// PLL0 frequency: 479.985664 MHz
		// CPU clock 120 MHz. PLL1 not used for USB.
		// 1. Disconnect PLL0 with one feed sequence if PLL0 is already connected
		if(PLL0CON & 0x00000002){
			// already connected
			PLL0CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 2. Disable PLL0 with one feed sequence.
		if(PLL0CON & 0x00000001){
			// already enabled
			PLL0CON &= ~0x00000001; // disable
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 3. Change the CPU Clock Divider setting to speed up operation without PLL0, if desired.
		// unused
		// 4. Write to the Clock Source Selection Control register to change the clock source if needed.
		CLKSRCSEL = 0x00000002; 	// Selects the RTC oscillator as the PLL0 clock source.
		// 5. Write to the PLL0CFG and make it effective with one feed sequence. The PLL0CFG
		// can only be updated when PLL0 is disabled.
		PLL0CFG = 0x00013937;	// M=14648, N=2 -> Fcco= 479.985664 MHz
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 6. Enable PLL0 with one feed sequence.
		PLL0CON = 0x00000001;
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 7. Change the CPU Clock Divider setting for the operation with PLL0. It is critical to do
		// this before connecting PLL0.
		CCLKCFG = 0x00000003;  	// divide by 4 from 480 to 120 MHz.
		// 8.Wait for PLL0 to achieve lock  200/32768 = 6.1ms
		// ????? delay affects OCD debugger????
		if(DEBUGGING != 1){
			for(i = 0; i< 30; i++){	// P1[10] unused pin
				FIO1SET = 0x00000400;
				FIO1CLR = 0x00000400;
				FIO1SET = 0x00000400;
			}
		}
		/*
		interruptsEnable(1);	// re-enable interrupts so that we can use the system tick
		delay_ms(6);
		interruptsEnable(0);	// disable interrupts
		*/

		// 9. Connect PLL0 with one feed sequence.
		PLL0CON |= 0x00000002; // connect
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 10. PLL1 (USB) disconnect
		if(PLL1CON & 0x00000002){
			// already connected
			PLL1CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 11. Disable PLL1 with one feed sequence.
		if(PLL1CON & 0x00000001){
			// already enabled
			PLL1CON &= ~0x00000001; // disable
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 12. Change the USB Clock Divider setting
		USBCLKCFG = 0x00000009; // divide by 10 from 480 MHz to 48 MHz

		// remember the CPU frequency for this profile
		cpuFreq = 120000000;
		// setSystemTimer
		setSystemTimer();
		// re-enable interrupts

	} else if(profile_no == 2){
		// profile 1:  PLL0 clock source = 20 MHz oscillator clock
		// PLL0 frequency: 480.00 MHz
		// CPU clock 120 MHz. PLL1 not used for USB.

		// enable oscillator
		SCS = 0x00000030;
			// main oscillator enabled, range 15-25 MHz

		// 1. Disconnect PLL0 with one feed sequence if PLL0 is already connected
		if(PLL0CON & 0x00000002){
			// already connected
			PLL0CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 2. Disable PLL0 with one feed sequence.
		if(PLL0CON & 0x00000001){
			// already enabled
			PLL0CON &= ~0x00000001; // disable
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 3. Change the CPU Clock Divider setting to speed up operation without PLL0, if desired.
		// unused
		// 4. Write to the Clock Source Selection Control register to change the clock source if needed.
		CLKSRCSEL = 0x00000001; 	// Selects the 20 MHz oscillator clock as the PLL0 clock source.
		// 5. Write to the PLL0CFG and make it effective with one feed sequence. The PLL0CFG
		// can only be updated when PLL0 is disabled.
		PLL0CFG = 0x0000000B;	// M=12, N=1 -> Fcco= 480.000 MHz
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 6. Enable PLL0 with one feed sequence.
		PLL0CON = 0x00000001;
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 7. Change the CPU Clock Divider setting for the operation with PLL0. It is critical to do
		// this before connecting PLL0.
		CCLKCFG = 0x00000003;  	// divide by 4 from 480 to 120 MHz.
		// 8.Wait for PLL0 to achieve lock  500us (we are still using the internal 4 MHz clock)
		toggle2(120);
		// 9. Connect PLL0 with one feed sequence.
		PLL0CON |= 0x00000002; // connect
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 10. PLL1 (USB) disconnect
		if(PLL1CON & 0x00000002){
			// already connected
			PLL1CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 11. Disable PLL1 with one feed sequence.
		if(PLL1CON & 0x00000001){
			// already enabled
			PLL1CON &= ~0x00000001; // disable
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 12. Change the USB Clock Divider setting
		USBCLKCFG = 0x00000009; // divide by 10 from 480 MHz to 48 MHz

		// remember the CPU frequency for this profile
		cpuFreq = 120000000;
		// setSystemTimer
		setSystemTimer();
		// re-enable interrupts

	} else if(profile_no == 4){
		// profile 1:  PLL0 clock source = 13 MHz oscillator clock
		// PLL0 frequency: 480.00 MHz
		// CPU clock 120 MHz. PLL1 not used for USB.
		// M=240, N=13, CPU divide 4

		// enable oscillator
		SCS = 0x00000020;
			// main oscillator enabled, range 1-20 MHz

		// 1. Disconnect PLL0 with one feed sequence if PLL0 is already connected
		if(PLL0CON & 0x00000002){
			// already connected
			PLL0CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 2. Disable PLL0 with one feed sequence.
		if(PLL0CON & 0x00000001){
			// already enabled
			PLL0CON &= ~0x00000001; // disable
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 3. Change the CPU Clock Divider setting to speed up operation without PLL0, if desired.
		// unused
		// 4. Write to the Clock Source Selection Control register to change the clock source if needed.
		CLKSRCSEL = 0x00000001; 	// Selects the 13 MHz oscillator clock as the PLL0 clock source.
		// 5. Write to the PLL0CFG and make it effective with one feed sequence. The PLL0CFG
		// can only be updated when PLL0 is disabled.
		PLL0CFG = 0x000C00EF;	// M=240, N=13 -> Fcco= 480.000 MHz
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 6. Enable PLL0 with one feed sequence.
		PLL0CON = 0x00000001;
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 7. Change the CPU Clock Divider setting for the operation with PLL0. It is critical to do
		// this before connecting PLL0.
		CCLKCFG = 0x00000003;  	// divide by 4 from 480 to 120 MHz.
		// 8.Wait for PLL0 to achieve lock  500us (we are still using the internal 4 MHz clock)
		toggle2(120);
		// 9. Connect PLL0 with one feed sequence.
		PLL0CON |= 0x00000002; // connect
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 10. PLL1 (USB) disconnect
		if(PLL1CON & 0x00000002){
			// already connected
			PLL1CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 11. Disable PLL1 with one feed sequence.
		if(PLL1CON & 0x00000001){
			// already enabled
			PLL1CON &= ~0x00000001; // disable
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 12. Change the USB Clock Divider setting
		USBCLKCFG = 0x00000009; // divide by 10 from 480 MHz to 48 MHz

		// remember the CPU frequency for this profile
		cpuFreq = 120000000;
		// setSystemTimer
		setSystemTimer();
		// re-enable interrupts

	} else if(profile_no == 5){
		// profile 1:  PLL0 clock source = 19.2 MHz oscillator clock
		// PLL0 frequency: 480.00 MHz
		// CPU clock 120 MHz. PLL1 not used for USB.
		// M=25, N=2, CPU divide 4

		// enable oscillator
		SCS = 0x00000030;
			// main oscillator enabled, range 15-25 MHz

		// 1. Disconnect PLL0 with one feed sequence if PLL0 is already connected
		if(PLL0CON & 0x00000002){
			// already connected
			PLL0CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 2. Disable PLL0 with one feed sequence.
		if(PLL0CON & 0x00000001){
			// already enabled
			PLL0CON &= ~0x00000001; // disable
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 3. Change the CPU Clock Divider setting to speed up operation without PLL0, if desired.
		// unused
		// 4. Write to the Clock Source Selection Control register to change the clock source if needed.
		CLKSRCSEL = 0x00000001; 	// Selects the 19.2 MHz oscillator clock as the PLL0 clock source.
		// 5. Write to the PLL0CFG and make it effective with one feed sequence. The PLL0CFG
		// can only be updated when PLL0 is disabled.
		PLL0CFG = 0x00010018;	// M=25, N=2 -> Fcco= 480.000 MHz
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 6. Enable PLL0 with one feed sequence.
		PLL0CON = 0x00000001;
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 7. Change the CPU Clock Divider setting for the operation with PLL0. It is critical to do
		// this before connecting PLL0.
		CCLKCFG = 0x00000003;  	// divide by 4 from 480 to 120 MHz.
		// 8.Wait for PLL0 to achieve lock  500us (we are still using the internal 4 MHz clock)
		toggle2(120);
		// 9. Connect PLL0 with one feed sequence.
		PLL0CON |= 0x00000002; // connect
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 10. PLL1 (USB) disconnect
		if(PLL1CON & 0x00000002){
			// already connected
			PLL1CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 11. Disable PLL1 with one feed sequence.
		if(PLL1CON & 0x00000001){
			// already enabled
			PLL1CON &= ~0x00000001; // disable
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 12. Change the USB Clock Divider setting
		USBCLKCFG = 0x00000009; // divide by 10 from 480 MHz to 48 MHz

		// remember the CPU frequency for this profile
		cpuFreq = 120000000;
		// setSystemTimer
		setSystemTimer();
		// re-enable interrupts
	} else if(profile_no == 6){
		// profile 1:  PLL0 clock source = 24 MHz crystal
		// PLL0 frequency: 480.00 MHz
		// CPU clock 120 MHz. PLL1 not used for USB.
		// M=, N=, CPU divide 4

		// enable oscillator
		SCS = 0x00000030;
			// main oscillator enabled, range 15-25 MHz

		// 1. Disconnect PLL0 with one feed sequence if PLL0 is already connected
		if(PLL0CON & 0x00000002){
			// already connected
			PLL0CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 2. Disable PLL0 with one feed sequence.
		if(PLL0CON & 0x00000001){
			// already enabled
			PLL0CON &= ~0x00000001; // disable
			// feed sequence
			PLL0FEED = 0x000000AA;
			PLL0FEED = 0x00000055;
		}
		// 3. Change the CPU Clock Divider setting to speed up operation without PLL0, if desired.
		// unused
		// 4. Write to the Clock Source Selection Control register to change the clock source if needed.
		CLKSRCSEL = 0x00000001; 	// Selects the 24 MHz oscillator clock as the PLL0 clock source.
		// 5. Write to the PLL0CFG and make it effective with one feed sequence. The PLL0CFG
		// can only be updated when PLL0 is disabled.
		PLL0CFG = 0x00000009;	// M=10, N=1 -> Fcco= 480.000 MHz
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 6. Enable PLL0 with one feed sequence.
		PLL0CON = 0x00000001;
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 7. Change the CPU Clock Divider setting for the operation with PLL0. It is critical to do
		// this before connecting PLL0.
		CCLKCFG = 0x00000003;  	// divide by 4 from 480 to 120 MHz.
		// 8.Wait for PLL0 to achieve lock  500us (we are still using the internal 4 MHz clock)
		toggle2(120);
		// 9. Connect PLL0 with one feed sequence.
		PLL0CON |= 0x00000002; // connect
		// feed sequence
		PLL0FEED = 0x000000AA;
		PLL0FEED = 0x00000055;
		// 10. PLL1 (USB) disconnect
		if(PLL1CON & 0x00000002){
			// already connected
			PLL1CON &= ~0x00000002; // disconnect
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 11. Disable PLL1 with one feed sequence.
		if(PLL1CON & 0x00000001){
			// already enabled
			PLL1CON &= ~0x00000001; // disable
			// feed sequence
			PLL1FEED = 0x000000AA;
			PLL1FEED = 0x00000055;
		}
		// 12. Change the USB Clock Divider setting
		USBCLKCFG = 0x00000009; // divide by 10 from 480 MHz to 48 MHz

		// remember the CPU frequency for this profile
		cpuFreq = 120000000;
		// setSystemTimer
		setSystemTimer();
		// re-enable interrupts
	}
	interruptsEnable(1);	// re-enable interrupts

	return;

}

void setSystemTimer(void){
	// adjust the system time so that it keeps a 10ms time interval
	// Call this function whenever the clock profile is changed.

	// Chapter 23: LPC17xx System Tick Timer
	STCTRL = 0x00000007;  // enable + interrupt enable + CPU clock as source
	STRELOAD = (unsigned long)((cpuFreq/100) -1);
	return;
}

void interruptsEnable(int en){
	// disable (en = 0) or enable (en <> 0) interrupts
	// UM10360	Chapter 6: LPC17xx Nested Vectored Interrupt Controller (NVIC)	5.1 Interrupt Set-Enable Register 0 register

	if(en){
		ISER0 = 0x01200000;	// peripherals using interrupts: USB, GPIO
		ISER1 = 0x00000000;
		asm volatile ("cpsie i");
			// CPSIE I is a assembly instruction to enable the priority configurable interrupts
	} else {
		ICER0 = 0x01200000;	// peripherals using interrupts: USB, GPIO
		ICER1 = 0x00000000;
		asm volatile ("cpsid i");
			// To turn off the priority configurable interrupts
	}
	return;
}

