/*-------------------------------------------------------------
 *  Copyright (c) 2013, MSS ComBlock. All rights reserved.
 *
 * 	Filename: ssp.c
 *  Last modified: 6/24/13
 *  Authors:
 *  	Alain Zarembowitch / MSS ComBlock
 *  Version: 4
 *  Inheritance: N/A
 *  Description: SSP configurations.
 * -------------------------------------------------------------*/
#include "main.h"
#include "ssp.h"
#include "vectors.h"
#include "lpc_types.h"

// global
//tbd

void ssp1_init(int enable) {
	// configure SSP1 as SPI slave, CPOL = 0, CPHA = 0, 16-bit frame, no interrupts, no DMA

	if(!enable){
		// Power Control for Peripherals register
		PCONP &= ~0x00000400;
		return;
	} else {
		// Power Control for Peripherals register
		PCONP |= 0x00000400;
		// Peripheral Clock Selection CCLK (here 120 MHz)
		PCLKSEL0 = (PCLKSEL0 & ~0x00300000) | 0x00100000;
		// SSP1 pins P0[6,7,8,9]
		PINSEL0 = (PINSEL0 & ~0x000FF000) | 0x000AA000;
		// SCLK pd, SSEL pu, MOSI don't care
		PINMODE0 = (PINMODE0 & ~0x000FF000) | 0x0008C000;
		// pin direction (slave)
		// FIOxDIR  port direction. 0 = input, 1 = output
		FIO0DIR = (FIO0DIR & ~0x000003C0) | 0x00000100;

		// prescaler (in theory not needed as this is a slave)
		SSP1CPSR = 0x00000002;
		// SSP Control Register 0.
		// 16-bit transfer,SPI, CPOL=0, CPHA=0, 10 MHz (if master)
		SSP1CR0 = 0x0000050F;
		// no interrupts
		SSP1IMSC = 0;
		// no DMA
		SSP1DMACR = 0;

		// SSP Control Register 1 (write last, once all other control registers defined).
		// slave, SSP enabled
		SSP1CR1 = 0x00000006;

	}
	return;
}

// returns -1 if no data, or 0x0000word when a 16-bit word is read
int ssp1_rxFIFO_poll(void){
	// returns 16-bit word
	if(SSP1SR & 0x04){
		// RNE bit high: receive FIFO Not Empty
		return SSP1DR;
	} else {
		// nothing read
		return -1;
	}
}

// write 16-bit word to SSP1 if there is room in the FIFO.
// returns 0 if success, otherwise, return a negative error.
int ssp1_txFIFO_write(int tx_word){
	// check status first
	if(SSP1SR & 0x02){
		// TFN bit high: transmit FIFO not full. OK to send another word
		SSP1DR = tx_word & 0x0000FFFF;
		return 0;
	} else {
		// tx FIFO is full
		return -1;
	}
}
